'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.default = initAppManager;

var _pathToRegexp = require('path-to-regexp');

var _pathToRegexp2 = _interopRequireDefault(_pathToRegexp);

var _slotFinder = require('slot-finder');

var _slotFinder2 = _interopRequireDefault(_slotFinder);

var _constants = require('./constants');

var _decorateHistory = require('./utils/decorate-history');

var _decorateHistory2 = _interopRequireDefault(_decorateHistory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initAppManager(w, html5History) {
  var _class, _temp, _initialiseProps;

  var history = (0, _decorateHistory2.default)(w, html5History);

  return _temp = _class = function () {
    (0, _createClass3.default)(AppManager, null, [{
      key: '_getBrowserPath',
      value: function _getBrowserPath() {
        var browserState = history.getState();
        return (browserState.hash || '').split('?')[0];
      }
    }, {
      key: '_timeout',
      value: function _timeout() {
        var timeout = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _constants.DEFAULT_TIMEOUT;

        return new _promise2.default(function (resolve) {
          return setTimeout(resolve, timeout);
        });
      }
    }, {
      key: 'triggerEvent',
      value: function triggerEvent(eventTitle, data) {
        history.events.trigger(eventTitle, data);
      }
    }, {
      key: 'bindEvent',
      value: function bindEvent(eventTitle, callback) {
        history.events.bind(eventTitle, callback);
      }
    }, {
      key: '_triggerError',
      value: function _triggerError(title, id, err) {
        AppManager.triggerEvent(AppManager.eventTitles.ERROR, { title: title, id: id, err: err });
      }
    }]);

    function AppManager(config, options) {
      (0, _classCallCheck3.default)(this, AppManager);

      _initialiseProps.call(this);

      if (!config) {
        throw new Error('Config not specified. Aborting.');
      }

      if (!w.document || !w.location) {
        throw new Error('Invalid window object. Aborting.');
      }

      var apps = config.apps,
          slots = config.slots,
          scripts = config.scripts;


      this._apps = apps;
      this._slots = slots;
      this._options = options || {};
      this._currentAppName = null;
      this._initialised = false;
      this._status = this.constructor.appStatuses.DEFAULT;
      this._scripts = (0, _keys2.default)(scripts || {}).reduce(function (mangedScripts, scriptName) {
        return scripts[scriptName].managed ? (0, _assign2.default)({}, mangedScripts, (0, _defineProperty3.default)({}, scriptName, scripts[scriptName])) : mangedScripts;
      }, {});

      this._pathCheckers = (0, _keys2.default)(apps || {}).reduce(function (appPathCheckers, appName) {
        var appPath = apps[appName].appPath;


        if (!appPath || typeof appPath !== 'string') {
          return appPathCheckers;
        }

        return (0, _assign2.default)({}, appPathCheckers, (0, _defineProperty3.default)({}, appName, (0, _pathToRegexp2.default)(appPath)));
      }, {});

      var browserTitle = w.document.title;
      var browserPath = w.location.pathname;
      var browserQuery = w.location.search;

      history.replaceState({}, browserTitle, '' + browserPath + browserQuery);
      history.events.bind(this.constructor.eventTitles.HISTORY_STATE_CHANGE, this.onStateChange);
    }

    (0, _createClass3.default)(AppManager, [{
      key: '_getSlotScripts',
      value: function _getSlotScripts(appName) {
        var _this = this;

        var appScripts = this._apps[appName].scripts.map(function (scriptName) {
          return _this._scripts[scriptName];
        });
        var emptySlots = (0, _keys2.default)(this._slots).reduce(function (empty, slotName) {
          return (0, _assign2.default)({}, empty, (0, _defineProperty3.default)({}, slotName, null));
        }, {});

        return (0, _slotFinder2.default)(emptySlots, appScripts);
      }
    }, {
      key: '_goToPath',
      value: function _goToPath() {
        history.events.unbind(this.constructor.eventTitles.HISTORY_STATE_CHANGE);

        var newPath = this.constructor._getBrowserPath();
        w.location.href = newPath; // eslint-disable-line no-param-reassign
      }
    }]);
    return AppManager;
  }(), _class.appStatuses = _constants.appStatuses, _class.eventTitles = _constants.eventTitles, _initialiseProps = function _initialiseProps() {
    var _this2 = this;

    this._setStatus = function (newStatus) {
      if (_this2._status !== newStatus) {
        _this2._status = newStatus;
        history.events.trigger(_this2.constructor.eventTitles.STATUS_CHANGE, newStatus);
      }
    };

    this._getAppFromPath = function () {
      var path = _this2.constructor._getBrowserPath();

      if (!path || typeof path !== 'string') {
        _this2.constructor._triggerError('get_app_from_path.no_path');
        return null;
      }

      return (0, _keys2.default)(_this2._pathCheckers).find(function (appName) {
        var pathChecker = _this2._pathCheckers[appName];

        if (!pathChecker) {
          _this2.constructor._triggerError('get_app_from_path.invalid_appPath', appName);
          return false;
        }

        return pathChecker.exec(path);
      });
    };

    this._getSlotElement = function (slotName) {
      var slot = _this2._slots[slotName];

      if (!slot) {
        _this2.constructor._triggerError('get_element.no_slot', slotName);
        return null;
      }

      var element = w.document.querySelector('.' + slot.elementClass);

      if (!element) {
        _this2.constructor._triggerError('get_element.no_element', slotName);
        return null;
      }

      return element;
    };

    this._getAppScript = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(appScriptName) {
        var fragmentScript, newScript;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (appScriptName) {
                  _context2.next = 3;
                  break;
                }

                _this2.constructor._triggerError('get_script.no_script_name');
                return _context2.abrupt('return', null);

              case 3:
                if (!(_this2._appScripts && _this2._appScripts[appScriptName])) {
                  _context2.next = 5;
                  break;
                }

                return _context2.abrupt('return', _this2._appScripts[appScriptName]());

              case 5:
                fragmentScript = _this2._scripts[appScriptName];

                if (fragmentScript) {
                  _context2.next = 9;
                  break;
                }

                _this2.constructor._triggerError('get_script.missing', appScriptName);
                return _context2.abrupt('return', null);

              case 9:
                if (!(typeof fragmentScript.load !== 'function')) {
                  _context2.next = 12;
                  break;
                }

                _this2.constructor._triggerError('get_script.not_function', appScriptName);
                return _context2.abrupt('return', null);

              case 12:
                _context2.prev = 12;
                _context2.next = 15;
                return _promise2.default.race([fragmentScript.load(), _this2.constructor._timeout(_this2._options.importTimeout)]);

              case 15:
                newScript = _context2.sent;

                if (newScript) {
                  _context2.next = 19;
                  break;
                }

                _this2.constructor._triggerError('get_script.no_file', appScriptName);
                return _context2.abrupt('return', null);

              case 19:
                if ('version' in newScript) {
                  _context2.next = 22;
                  break;
                }

                _this2.constructor._triggerError('get_script.invalid_script', appScriptName);
                return _context2.abrupt('return', null);

              case 22:

                _this2._appScripts = (0, _assign2.default)({}, _this2._appScripts, (0, _defineProperty3.default)({}, appScriptName, (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
                  return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          return _context.abrupt('return', newScript);

                        case 1:
                        case 'end':
                          return _context.stop();
                      }
                    }
                  }, _callee, _this2);
                }))));

                return _context2.abrupt('return', newScript);

              case 26:
                _context2.prev = 26;
                _context2.t0 = _context2['catch'](12);

                _this2.constructor._triggerError('get_script.load', appScriptName, _context2.t0);
                return _context2.abrupt('return', null);

              case 30:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this2, [[12, 26]]);
      }));

      return function (_x2) {
        return _ref.apply(this, arguments);
      };
    }();

    this._getPageScript = function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(slotScripts, slotName) {
        var slotScriptName, appScript;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                slotScriptName = slotScripts[slotName];

                if (slotScriptName) {
                  _context3.next = 3;
                  break;
                }

                return _context3.abrupt('return', null);

              case 3:
                _context3.next = 5;
                return _this2._getAppScript(slotScriptName);

              case 5:
                appScript = _context3.sent;
                return _context3.abrupt('return', appScript ? { script: appScript, slot: slotName, name: slotScriptName } : null);

              case 7:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this2);
      }));

      return function (_x3, _x4) {
        return _ref3.apply(this, arguments);
      };
    }();

    this._getPageScripts = function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(appName) {
        var slotScripts, appScriptImportPromises;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                slotScripts = _this2._getSlotScripts(appName);

                if (!((typeof slotScripts === 'undefined' ? 'undefined' : (0, _typeof3.default)(slotScripts)) !== 'object')) {
                  _context4.next = 4;
                  break;
                }

                _this2.constructor._triggerError('get_page_scripts.get', appName);
                return _context4.abrupt('return', null);

              case 4:
                appScriptImportPromises = (0, _keys2.default)(slotScripts).map(function (slotName) {
                  return _this2._getPageScript(slotScripts, slotName);
                });
                return _context4.abrupt('return', _promise2.default.all(appScriptImportPromises));

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, _this2);
      }));

      return function (_x5) {
        return _ref4.apply(this, arguments);
      };
    }();

    this._hydrateScript = function (appName) {
      return function () {
        var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(appScript) {
          var slotElement, newApp, script;
          return _regenerator2.default.wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  if (appScript.slot) {
                    _context5.next = 3;
                    break;
                  }

                  _this2.constructor._triggerError('hydrate.missing_slot', appScript.name || appName);
                  return _context5.abrupt('return', false);

                case 3:
                  slotElement = _this2._getSlotElement(appScript.slot);

                  if (slotElement) {
                    _context5.next = 7;
                    break;
                  }

                  _this2.constructor._triggerError('hydrate.missing_element', appScript.slot);
                  return _context5.abrupt('return', false);

                case 7:
                  newApp = _this2._apps[appName];

                  if (newApp) {
                    _context5.next = 11;
                    break;
                  }

                  _this2.constructor._triggerError('hydrate.invalid_app', appName);
                  return _context5.abrupt('return', false);

                case 11:
                  if (appScript.script) {
                    _context5.next = 14;
                    break;
                  }

                  _this2.constructor._triggerError('hydrate.missing_script', appName);
                  return _context5.abrupt('return', false);

                case 14:
                  if (!(typeof appScript.script.hydrate !== 'function')) {
                    _context5.next = 17;
                    break;
                  }

                  _this2.constructor._triggerError('hydrate.invalid_script', appScript.name || appName);
                  return _context5.abrupt('return', false);

                case 17:
                  script = appScript.script;
                  _context5.prev = 18;
                  _context5.next = 21;
                  return script.hydrate(slotElement, history, newApp);

                case 21:
                  return _context5.abrupt('return', true);

                case 24:
                  _context5.prev = 24;
                  _context5.t0 = _context5['catch'](18);

                  _this2.constructor._triggerError('hydrate.run_script', appScript.name, _context5.t0);
                  return _context5.abrupt('return', false);

                case 28:
                case 'end':
                  return _context5.stop();
              }
            }
          }, _callee5, _this2, [[18, 24]]);
        }));

        return function (_x6) {
          return _ref5.apply(this, arguments);
        };
      }();
    };

    this._unmountScript = function (appName) {
      return function () {
        var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(appScript) {
          var slotElement, newApp, script;
          return _regenerator2.default.wrap(function _callee6$(_context6) {
            while (1) {
              switch (_context6.prev = _context6.next) {
                case 0:
                  slotElement = _this2._getSlotElement(appScript.slot);

                  if (slotElement) {
                    _context6.next = 4;
                    break;
                  }

                  _this2.constructor._triggerError('unmount.missing', appScript.name);
                  return _context6.abrupt('return', false);

                case 4:
                  newApp = _this2._apps[appName];

                  if (newApp) {
                    _context6.next = 8;
                    break;
                  }

                  _this2.constructor._triggerError('unmount.invalid_app', appName);
                  return _context6.abrupt('return', false);

                case 8:
                  if (appScript.script) {
                    _context6.next = 11;
                    break;
                  }

                  _this2.constructor._triggerError('unmount.missing_script', appName);
                  return _context6.abrupt('return', false);

                case 11:
                  if (!(typeof appScript.script.unmount !== 'function')) {
                    _context6.next = 14;
                    break;
                  }

                  _this2.constructor._triggerError('unmount.invalid_script', appScript.name || appName);
                  return _context6.abrupt('return', false);

                case 14:
                  script = appScript.script;
                  _context6.prev = 15;
                  _context6.next = 18;
                  return script.unmount(slotElement, history, newApp);

                case 18:
                  return _context6.abrupt('return', true);

                case 21:
                  _context6.prev = 21;
                  _context6.t0 = _context6['catch'](15);

                  _this2.constructor._triggerError('unmount.error', appScript.name || appName, _context6.t0);
                  return _context6.abrupt('return', false);

                case 25:
                case 'end':
                  return _context6.stop();
              }
            }
          }, _callee6, _this2, [[15, 21]]);
        }));

        return function (_x7) {
          return _ref6.apply(this, arguments);
        };
      }();
    };

    this._mountScript = function (appName) {
      return function () {
        var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(appScript) {
          var slotElement, newApp, script;
          return _regenerator2.default.wrap(function _callee7$(_context7) {
            while (1) {
              switch (_context7.prev = _context7.next) {
                case 0:
                  slotElement = _this2._getSlotElement(appScript.slot);

                  if (slotElement) {
                    _context7.next = 4;
                    break;
                  }

                  _this2.constructor._triggerError('mount.missing', appScript.name || appName);
                  return _context7.abrupt('return', false);

                case 4:
                  newApp = _this2._apps[appName];

                  if (newApp) {
                    _context7.next = 8;
                    break;
                  }

                  _this2.constructor._triggerError('mount.invalid_app', appName);
                  return _context7.abrupt('return', false);

                case 8:
                  if (appScript.script) {
                    _context7.next = 11;
                    break;
                  }

                  _this2.constructor._triggerError('mount.missing_script', appName);
                  return _context7.abrupt('return', false);

                case 11:
                  if (!(typeof appScript.script.mount !== 'function')) {
                    _context7.next = 14;
                    break;
                  }

                  _this2.constructor._triggerError('mount.invalid_script', appScript.name || appName);
                  return _context7.abrupt('return', false);

                case 14:
                  script = appScript.script;
                  _context7.prev = 15;
                  _context7.next = 18;
                  return script.mount(slotElement, history, newApp);

                case 18:
                  return _context7.abrupt('return', true);

                case 21:
                  _context7.prev = 21;
                  _context7.t0 = _context7['catch'](15);

                  _this2.constructor._triggerError('mount.error', appScript.name || appName, _context7.t0);
                  return _context7.abrupt('return', false);

                case 25:
                case 'end':
                  return _context7.stop();
              }
            }
          }, _callee7, _this2, [[15, 21]]);
        }));

        return function (_x8) {
          return _ref7.apply(this, arguments);
        };
      }();
    };

    this._stateChangeScript = function (appName) {
      return function () {
        var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(appScript) {
          var updateApp, script;
          return _regenerator2.default.wrap(function _callee8$(_context8) {
            while (1) {
              switch (_context8.prev = _context8.next) {
                case 0:
                  updateApp = _this2._apps[appName];

                  if (updateApp) {
                    _context8.next = 4;
                    break;
                  }

                  _this2.constructor._triggerError('state_change.invalid_app', appName);
                  return _context8.abrupt('return', false);

                case 4:
                  if (appScript.script) {
                    _context8.next = 7;
                    break;
                  }

                  _this2.constructor._triggerError('state_change.missing_script', appName);
                  return _context8.abrupt('return', false);

                case 7:
                  if (!(typeof appScript.script.onStateChange !== 'function')) {
                    _context8.next = 10;
                    break;
                  }

                  _this2.constructor._triggerError('state_change.invalid_script', appScript.name || appName);
                  return _context8.abrupt('return', false);

                case 10:
                  script = appScript.script;
                  _context8.prev = 11;
                  _context8.next = 14;
                  return script.onStateChange(history, updateApp);

                case 14:
                  return _context8.abrupt('return', true);

                case 17:
                  _context8.prev = 17;
                  _context8.t0 = _context8['catch'](11);

                  _this2.constructor._triggerError('state_change.error', appScript.name || appName, _context8.t0);
                  return _context8.abrupt('return', false);

                case 21:
                case 'end':
                  return _context8.stop();
              }
            }
          }, _callee8, _this2, [[11, 17]]);
        }));

        return function (_x9) {
          return _ref8.apply(this, arguments);
        };
      }();
    };

    this._hydratePage = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9() {
      var currentAppName, pageScripts, scripts, hydratePromises, hydrateSuccess;
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              currentAppName = _this2._currentAppName;

              if (currentAppName) {
                _context9.next = 4;
                break;
              }

              _this2.constructor._triggerError('hydrate_page.missing_route');
              return _context9.abrupt('return', false);

            case 4:
              _context9.prev = 4;
              _context9.next = 7;
              return _this2._getPageScripts(currentAppName);

            case 7:
              pageScripts = _context9.sent;

              if (!(!Array.isArray(pageScripts) || !pageScripts.every(Boolean))) {
                _context9.next = 10;
                break;
              }

              return _context9.abrupt('return', false);

            case 10:
              scripts = pageScripts;
              hydratePromises = scripts.map(_this2._hydrateScript(currentAppName));
              _context9.next = 14;
              return _promise2.default.all(hydratePromises);

            case 14:
              hydrateSuccess = _context9.sent;
              return _context9.abrupt('return', hydrateSuccess.every(Boolean));

            case 18:
              _context9.prev = 18;
              _context9.t0 = _context9['catch'](4);

              _this2.constructor._triggerError('hydrate_page.get_scripts', currentAppName, _context9.t0);
              return _context9.abrupt('return', false);

            case 22:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, _this2, [[4, 18]]);
    }));

    this._updatePage = function () {
      var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(newAppName) {
        var oldAppName, _ref11, _ref12, oldAppScripts, newAppScripts, oldScripts, newScripts, oldAppScriptNames, newAppScriptNames, unmountPromises, mountPromises, updatePromises, unmountSuccess, mountSuccess, updateSuccess;

        return _regenerator2.default.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                oldAppName = _this2._currentAppName;

                if (oldAppName) {
                  _context10.next = 4;
                  break;
                }

                _this2.constructor._triggerError('update_page.no_current_app');
                return _context10.abrupt('return', false);

              case 4:
                _context10.next = 6;
                return _promise2.default.all([_this2._getPageScripts(oldAppName), _this2._getPageScripts(newAppName)]);

              case 6:
                _ref11 = _context10.sent;
                _ref12 = (0, _slicedToArray3.default)(_ref11, 2);
                oldAppScripts = _ref12[0];
                newAppScripts = _ref12[1];

                if (!(!Array.isArray(oldAppScripts) || !Array.isArray(newAppScripts) || !oldAppScripts.every(Boolean) || !newAppScripts.every(Boolean))) {
                  _context10.next = 12;
                  break;
                }

                return _context10.abrupt('return', false);

              case 12:
                oldScripts = oldAppScripts;
                newScripts = newAppScripts;
                oldAppScriptNames = oldScripts.map(function (_ref13) {
                  var name = _ref13.name;
                  return name;
                });
                newAppScriptNames = newScripts.map(function (_ref14) {
                  var name = _ref14.name;
                  return name;
                });
                unmountPromises = oldScripts.filter(function (appScript) {
                  return newAppScriptNames.indexOf(appScript.name) === -1;
                }).map(_this2._unmountScript(oldAppName));
                mountPromises = newScripts.filter(function (appScript) {
                  return oldAppScriptNames.indexOf(appScript.name) === -1;
                }).map(_this2._mountScript(newAppName));
                updatePromises = oldScripts.filter(function (appScript) {
                  return newAppScriptNames.indexOf(appScript.name) > -1;
                }).map(_this2._stateChangeScript(newAppName));
                _context10.next = 21;
                return _promise2.default.all(unmountPromises);

              case 21:
                unmountSuccess = _context10.sent;

                if (unmountSuccess.every(Boolean)) {
                  _context10.next = 24;
                  break;
                }

                return _context10.abrupt('return', false);

              case 24:
                _context10.next = 26;
                return _promise2.default.all(mountPromises);

              case 26:
                mountSuccess = _context10.sent;

                if (mountSuccess.every(Boolean)) {
                  _context10.next = 29;
                  break;
                }

                return _context10.abrupt('return', false);

              case 29:
                _context10.next = 31;
                return _promise2.default.all(updatePromises);

              case 31:
                updateSuccess = _context10.sent;

                if (updateSuccess.every(Boolean)) {
                  _context10.next = 34;
                  break;
                }

                return _context10.abrupt('return', false);

              case 34:
                return _context10.abrupt('return', true);

              case 35:
              case 'end':
                return _context10.stop();
            }
          }
        }, _callee10, _this2);
      }));

      return function (_x10) {
        return _ref10.apply(this, arguments);
      };
    }();

    this.onStateChange = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11() {
      var newAppName, updatePageSuccess;
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              if (_this2._initialised) {
                _context11.next = 2;
                break;
              }

              return _context11.abrupt('return', false);

            case 2:
              if (!(_this2._status === _this2.constructor.appStatuses.ERROR)) {
                _context11.next = 5;
                break;
              }

              _this2._goToPath();
              return _context11.abrupt('return', false);

            case 5:

              _this2._setStatus(_this2.constructor.appStatuses.LOADING);

              newAppName = _this2._getAppFromPath();

              if (newAppName) {
                _context11.next = 12;
                break;
              }

              _this2.constructor._triggerError('update_app.app_not_found');
              _this2._setStatus(_this2.constructor.appStatuses.ERROR);
              _this2._goToPath();
              return _context11.abrupt('return', false);

            case 12:
              _context11.prev = 12;
              _context11.next = 15;
              return _this2._updatePage(newAppName);

            case 15:
              updatePageSuccess = _context11.sent;

              if (updatePageSuccess) {
                _context11.next = 20;
                break;
              }

              _this2._setStatus(_this2.constructor.appStatuses.ERROR);
              _this2._goToPath();
              return _context11.abrupt('return', false);

            case 20:

              _this2._currentAppName = newAppName;
              _this2._setStatus(_this2.constructor.appStatuses.DEFAULT);

              return _context11.abrupt('return', true);

            case 25:
              _context11.prev = 25;
              _context11.t0 = _context11['catch'](12);

              _this2.constructor._triggerError('update_app', newAppName, _context11.t0);
              _this2._setStatus(_this2.constructor.appStatuses.ERROR);
              _this2._goToPath();
              return _context11.abrupt('return', false);

            case 31:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, _this2, [[12, 25]]);
    }));

    this._initError = function (type) {
      if (type) {
        _this2.constructor._triggerError('init.' + type);
      }

      _this2._setStatus(_this2.constructor.appStatuses.ERROR);
      history.events.unbind(_this2.constructor.eventTitles.HISTORY_STATE_CHANGE);
      return false;
    };

    this.init = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12() {
      var currentAppName, hydrateSuccess;
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _this2._setStatus(_this2.constructor.appStatuses.LOADING);

              if ((0, _keys2.default)(_this2._slots).every(function (slotName) {
                return _this2._getSlotElement(slotName);
              })) {
                _context12.next = 3;
                break;
              }

              return _context12.abrupt('return', _this2._initError('missing_elements'));

            case 3:
              currentAppName = _this2._getAppFromPath();

              if (currentAppName) {
                _context12.next = 6;
                break;
              }

              return _context12.abrupt('return', _this2._initError('no_route'));

            case 6:

              _this2._currentAppName = currentAppName;

              _context12.next = 9;
              return _this2._hydratePage();

            case 9:
              hydrateSuccess = _context12.sent;

              if (hydrateSuccess) {
                _context12.next = 12;
                break;
              }

              return _context12.abrupt('return', _this2._initError());

            case 12:

              _this2._initialised = true;
              _this2._setStatus(_this2.constructor.appStatuses.DEFAULT);

              return _context12.abrupt('return', true);

            case 15:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, _this2);
    }));
  }, _temp;
}